/* BEGIN COPYRIGHT BLOCK
 Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
 Copyright (C) 2005 Red Hat, Inc.
 All rights reserved.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation version
 2.1 of the License.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK */

/*
 * ux-curse.c: Routines shared among server setup programs.
 * 
 * These functions are unix-only and require curses.
 *
 * Rob McCool
 */

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "strings.h"
#include "ux-curse.h"

static int  _w_isendwin = 1;

static char header[80] = "Fedora Server Family Installation/Uninstallation";
static char title[80] = "Fedora Project";

void init_screen (const char *what_server, const char *caption)
{
   if (what_server)
      sprintf(header, "%s Installation/Uninstallation", what_server);

   if (caption)
      strcpy(title, caption);

/*
   initscr();
*/
}
   
void show_message(const char *msg, ...)
{
    char errbuf[TEMP_STRING_SIZE];
    va_list arg;

    va_start(arg, msg);
    vsnprintf(errbuf, sizeof(errbuf), msg, arg);
    va_end(arg);
    errbuf[sizeof(errbuf)-1] = 0;
    printw(errbuf);
    printw("Press any key to continue.\n");
    refresh();
    _w_isendwin = 0;
    getch();
}

void exit_message(const char *msg, ...)
{
    char errbuf[TEMP_STRING_SIZE];
    int x, y;
    va_list arg;

    va_start(arg, msg);
    vsnprintf(errbuf, sizeof(errbuf), msg, arg);
    va_end(arg);
    errbuf[sizeof(errbuf)-1] = 0;

    getyx(stdscr, y, x);

    endwin();
    fprintf(stderr, "\n%s\n", errbuf);
    exit(1);
}

/*
 * PVO - 1: Next
 *       0: Prev
 */ 

int bufFull = 0;
int cursey;
int cursex;
char tmpBuf[1024];
static int disableCtrlB = 0;

void enableScreenBack()
{
   disableCtrlB = 0;
}

void disableScreenBack()
{
   disableCtrlB = 1;
}
   
int grab_string_generic(char *t, int l, int do_echo, const char *deft)
{
    int tind, x, y, i, origy, origx;
    int rc = 1;

#if defined(SOLARIS)
    noecho();

    refresh();
    tind = 0;

    if (bufFull)
    {
       strcpy(t, tmpBuf);
       tind = strlen(t);
       origy = cursey;
       origx = cursex;
    }
    else
    {
       getyx(stdscr, origy, origx);
    }
    while(1) {
        getyx(stdscr, y, x);
        i = getch();
        if(i == -1)
            exit_message("Could not read your input. Exiting.\n");
        if(i == 13)      /* control-M */
            continue;
        if(i == 10) {    /* control-J */
            t[tind] = '\0';
            break;
        }
        if (i == 21) {   /* control-U */
            tind = 0;
            if (do_echo) {
	            while ( y > origy ) {
                  move( y--, 0 );
                  clrtoeol();
		         }
		         move(y, origx); clrtoeol();
	         }
            continue;
        }
        if (i == 2 && !disableCtrlB) { /* control-B */
           tind = 0;
           bufFull = 0;
           echo();
           return 0;
        }

        if ((i == 8) || (i == 127)) {    /* control-H  or delete */
            if(tind) {
               --tind;
               if (do_echo) {
		            if ( x ) {
		               move(y, x-1);
		            } else {
		               move( y, 0 );
		               clrtoeol();
#if defined(HPUX_64)
  		               move( y - 1, 200 -1 ); 
#else
  		               move( y - 1, stdscr->_maxx -1 ); 
#endif
		            }
		            clrtoeol();
		         }
            }
            else move(y, origx);  /* make sure we're not past the prompt */
            continue;
        }

        if (i == 9 && deft) {    /* TAB */
            strcpy(t, deft);
            tind = strlen(deft);
            if (do_echo) {
               move(y, origx); 
               clrtoeol();
               printw("%s", deft);
            }
            continue;
        }

        t[tind++] = (char) i;
        if (do_echo)
            printw("%c", (char)i);

        if(tind == l)
            exit_message("That's way too long. Exiting.\n");
    }
    getyx(stdscr, y, x);
    move(y+1, 0);

    echo();
#else
    if (!do_echo)
	noecho();

    refresh();
    tind = 0;

    if (bufFull)
    {
       strcpy(t, tmpBuf);
       tind = strlen(t);
       origy = cursey;
       origx = cursex;
    }
    else
    {
       getyx(stdscr, origy, origx);
    }
    while(1) {
        getyx(stdscr, y, x);
        i = getch();
        if(i == -1)
            exit_message("Could not read your input. Exiting.\n");
        if(i == 13)      /* control-M */
            continue;
        if(i == 10) {    /* control-J */
            t[tind] = '\0';
            break;
        }
        if (i == 21) {   /* control-U */
            tind = 0;
            if (do_echo) {
	        while ( y > origy ) {
		  move( y--, 0 );
		  clrtoeol();
		}
		move(y, origx); clrtoeol();
	    }
            continue;
        }
        if (i == 2 && !disableCtrlB) { /* control-B */
           tind = 0;
           bufFull = 0;
           if (!do_echo)
           {
              echo();
           }
           return 0;
        }

        if ((i == 8) || (i == 127)) {    /* control-H  or delete */
            if(tind) {
                --tind;
                if (do_echo) {
		  if ( x ) {
                    move(y, x-1);
		  } else {
		    move( y, 0 );
                    clrtoeol();
#if defined(HPUX_64)
  		    move( y - 1, 200 -1 ); 
#else
  		    move( y - 1, stdscr->_maxx -1 ); 
#endif
		  }
		  clrtoeol();
		}
            }
            else move(y, origx);  /* make sure we're not past the prompt */
            continue;
        }

        t[tind++] = (char) i;

        if(tind == l)
            exit_message("That's way too long. Exiting.\n");
    }
    getyx(stdscr, y, x);
    move(y+1, 0);

    if (!do_echo)
	echo();
#endif
    bufFull = 0;
    disableCtrlB = 0;
    return rc;
}


int grab_string(char *t, int l, const char *deft)
{
    return (grab_string_generic(t, l, 1, deft));
}


int grab_pw_string(char *t, int l, const char *deft)
{
    return grab_string_generic(t, l, 0, deft);
}


void print_centered(int width, const char *msg)
{
    int x, len = strlen(msg), blank = (width - len) / 2;
    for(x = 0; x < blank; x++) printw(" ");
    printw("%s\n", msg);
}

void print_oneplace(const char *msg)
{
    int x, y;

    getyx(stdscr, y , x);
    printw("%s", msg);
    move (y,x);
}

void new_page()
{
   int x, width;

#if defined(HPUX_64)
   width = 200;
#else
   width = stdscr->_maxx;
#endif

   clear();
   printw("\n");
   print_centered(width, title);
   print_centered(width, header);
   for(x = 0; x < width; ++x) printw("%c", '-');
   printw("\n\n");
   _w_isendwin = 0;
}

/*
 * Stupid wrapper for C++
 */
int w_refresh (void)
{
   refresh();
   _w_isendwin = 0;
}

int w_endwin(void)
{
   endwin();
   _w_isendwin = 1;
}

int w_initscr(void)
{
   if (newterm (getenv("TERM"), stdout, stdin) == NULL)
   {
      if (newterm ("ansi", stdout, stdin) == NULL)
      {
         if (newterm ("vt100", stdout, stdin) == NULL)
         {
            printf("Can't initialize curses screen."
                "Possible cause: your TERM is not set correctly\n");
         }
      }
   }
   _w_isendwin = 0;
}

int w_isendwin(void)
{
   return _w_isendwin;
}

int
w_ask_yes_no(int useDefault)
{
   char            buf[5];

   grab_string(buf, sizeof(buf), NULL);
   if (buf[0] == '\0' || buf[0] == '\n')
      return useDefault;
   else if (buf[0] != 'y' && buf[0] != 'Y')
      return NO;
   else
      return YES;
}

/*
void stuff_buffer(const char *t)
{
   getyx (stdscr, cursey, cursex);
   if (t)
   {
      printw(t);
      strcpy(tmpBuf, t);
      bufFull = 1;
   }
} 
*/
